@@ -1,12 +1,13 @@
 // Copyright 2011 The Go Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style
 // license that can be found in the LICENSE file.
+
 package sync_test
 
 import (
-	. "sync"
-
+	"reflect"
 	"runtime"
+	. "sync"
 	"testing"
 	"time"
 )
@@ -251,7 +252,8 @@ func TestCondCopy(t *testing.T) {
 	}()
 	c := Cond{L: &Mutex{}}
 	c.Signal()
-	c2 := c
+	var c2 Cond
+	reflect.ValueOf(&c2).Elem().Set(reflect.ValueOf(&c).Elem()) // c2 := c, hidden from vet
 	c2.Signal()
 }
 
